<?php

/**
 * E2pdf Convert Helper
 * 
 * @copyright  Copyright 2017 https://e2pdf.com
 * @license    GPLv3
 * @version    1
 * @link       https://e2pdf.com
 * @since      1.24.05
 */
if (!defined('ABSPATH')) {
    die('Access denied.');
}

class Helper_E2pdf_Cache {

    private $helper;

    public function __construct() {
        $this->helper = Helper_E2pdf_Helper::instance();
    }

    public function clear($force = false) {
        $files = glob($this->helper->get('cache_dir') . '*', GLOB_MARK);
        $timeout = max(10, (int) get_option('e2pdf_cache_pdfs_timeout', '180'));
        foreach ($files as $file) {
            if (false === strpos($file, 'index.php') && false === strpos($file, '.htaccess') && is_file($file) && ($force || ((time() - filectime($file)) > $timeout))) {
                @unlink($file);
            }
        }
    }

    public function cache($cached_pdf = '', $request = array()) {
        if ($cached_pdf && get_option('e2pdf_cache_pdfs', '0') && !@file_exists($this->helper->get('cache_dir') . $cached_pdf)) {
            if (!isset($request['error']) && !empty($request['file'])) {
                file_put_contents($this->helper->get('cache_dir') . $cached_pdf, $request['file']);
            }
        }
    }

    public function request($cached_pdf = '') {
        $request = array();
        if ($cached_pdf && get_option('e2pdf_cache_pdfs', '0') && @file_exists($this->helper->get('cache_dir') . $cached_pdf)) {
            $request = array(
                'file' => @file_get_contents($this->helper->get('cache_dir') . $cached_pdf),
            );
            $this->clear();
        }
        return $request;
    }
}
